@@ -196,8 +196,9 @@ class DataIndex(MutableMapping):
 
 def _collect_dir(index, prefix, entry, path, fs):
     for root, dnames, fnames in fs.walk(path):
+        sub_prefix = fs.path.relparts(root, path) if root != path else ()
         for name in chain(dnames, fnames):
-            key = (*prefix, name)
+            key = (*prefix, *sub_prefix, name)
             entry_path = fs.path.join(root, name)
 
             # TODO: localfs.walk doesn't currently support detail=True,
@@ -221,10 +222,12 @@ def collect(index, path, fs):
 
         info = fs.info(entry_path)
 
-        entry.meta = Meta.from_info(info, fs.protocol)
+        fs_meta = Meta.from_info(info, fs.protocol)
+        if entry.meta != fs_meta:
+            entry.meta = Meta.from_info(info, fs.protocol)
+            entry.hash_info = None
         entry.fs = fs
         entry.path = entry_path
-        entry.hash_info = None
 
         if info["type"] == "file":
             continue
